INSTALLED_APPS = (
    'oauth_tokens',
)

OAUTH_TOKENS_FACEBOOK_CLIENT_ID = 439210362826994
OAUTH_TOKENS_FACEBOOK_CLIENT_SECRET = '02569c93d18e4bc6aa210e594af84652'
OAUTH_TOKENS_FACEBOOK_SCOPE = ['offline_access']
OAUTH_TOKENS_FACEBOOK_USERNAME = '+919665223715'
OAUTH_TOKENS_FACEBOOK_PASSWORD = 'github0vich'